// Copyright 2014 Miloš Vukov, BSD-style copyright and disclaimer apply
var Diploma;
(function (Diploma) {
    (function (Core) {
        /** Event aggregator */
        var EventAggregator = (function () {
            function EventAggregator() {
            }
            /**
            * Publish message with data.
            * @param topic Message topic.
            * @param range Publish range.
            * @param data Message data.
            */
            EventAggregator.Publish = function (topic, range, data) {
                switch (range) {
                    case 1 /* All */:
                        chrome.runtime.sendMessage({ Topic: topic, data: data });
                        break;
                    case 0 /* Local */:
                        amplify.publish(topic, data);
                        break;
                }
            };

            /**
            * Subscribe to message topic.
            * @param topic Message topic.
            * @param onMessage On message receive.
            */
            EventAggregator.Subscribe = function (topic, onMessage) {
                amplify.subscribe(topic, onMessage);
            };

            /** Initialize Event aggregator. */
            EventAggregator.Initialize = function () {
                // Message from background script.
                chrome.runtime.onMessage.addListener(EventAggregator.OnServerPublishMessage);
            };

            /** Publish message local via amplifyjs. */
            EventAggregator.OnServerPublishMessage = function (request, sender, sendResponse) {
                if (!request.Data)
                    amplify.publish(request.Topic);
                else
                    amplify.publish(request.Topic, request.Data);
                return true;
            };
            return EventAggregator;
        })();
        Core.EventAggregator = EventAggregator;

        /** Event aggregator hub. */
        var EventAggregatorHub = (function () {
            function EventAggregatorHub() {
            }
            /** Initialize event aggregator hub. */
            EventAggregatorHub.prototype.Initialize = function () {
                // Message from content script.
                chrome.runtime.onMessage.addListener(this.OnClientPublishMessage);
            };

            /** Dispatch message to all content scripts */
            EventAggregatorHub.prototype.OnClientPublishMessage = function (request, sender, sendResponse) {
                var fromTabId = sender.tab.id;

                // Get all tabs.
                chrome.tabs.query({}, function (tabs) {
                    for (var index = 0; index < tabs.length; index++) {
                        var id = tabs[index].id;
                        if (id != fromTabId) {
                            // Send only to other tabs.
                            chrome.tabs.sendMessage(id, request);
                        }
                    }
                });
                return true;
            };

            /** Get instance of event aggregator hub. */
            EventAggregatorHub.Default = function () {
                if (!EventAggregatorHub._defaultHub) {
                    EventAggregatorHub._defaultHub = new EventAggregatorHub();
                }
                return EventAggregatorHub._defaultHub;
            };
            return EventAggregatorHub;
        })();
        Core.EventAggregatorHub = EventAggregatorHub;

        /** Event aggregator publish range. */
        (function (PublishRange) {
            PublishRange[PublishRange["Local"] = 0] = "Local";
            PublishRange[PublishRange["All"] = 1] = "All";
        })(Core.PublishRange || (Core.PublishRange = {}));
        var PublishRange = Core.PublishRange;
    })(Diploma.Core || (Diploma.Core = {}));
    var Core = Diploma.Core;
})(Diploma || (Diploma = {}));
//# sourceMappingURL=EventAggregator.js.map
