// Copyright 2014 Miloš Vukov, BSD-style copyright and disclaimer apply
var Diploma;
(function (Diploma) {
    (function (Core) {
        /** Handle cursor */
        var Pointer = (function () {
            function Pointer() {
                this._pointerOffsetX = -20;
                this._pointerOffsetY = -20;
                this._currentType = 0 /* Pointer */;
            }
            /**
            * Move pointer.
            * @param x Pointer X position.
            * @param y Pointer Y position.
            */
            Pointer.prototype.MovePointer = function (x, y) {
                this._currentType = 0 /* Pointer */;

                var highlight = this.GetHighlight();

                highlight.hide();

                // Move to current position
                var pointer = this.GetPointer();
                var css = {
                    "left": (x + "px"),
                    "top": (y + "px")
                };
                pointer.css(css);
                pointer.show();
                pointer.offset(pointer.offset()); // FIX: Causes repaint.
            };

            /** Animate screen tap on highlight cursor. */
            Pointer.prototype.ScreenTap = function () {
                var _this = this;
                if (this.IsCursor(2 /* Highlight */)) {
                    this.Using(function () {
                        var highlight = _this.GetHighlight();
                        highlight.one('webkitAnimationEnd', function (event) {
                            $(event.target).removeClass("pointer-animations-click");
                        });

                        // Play screentap animation
                        highlight.addClass("pointer-animations-click");
                        //highlight.height(highlight.height()); // TODO: Milos - Preveri očino deluje brez prožanja (workarround) animacije
                    }, 1 /* ScreenTap */);
                }
            };

            /**
            * Change cursor and highlight element.
            * @param element JQuery element to highlight.
            */
            Pointer.prototype.Highlight = function (element) {
                if (!this.IsCursor(2 /* Highlight */)) {
                    this._currentType = 2 /* Highlight */;

                    // Set new type and hide current.
                    this.GetPointer().hide();
                    var highlight = this.GetHighlight();

                    // Get element width and height.
                    var elementPosition = element.offset();
                    var h = element.innerHeight();
                    var w = element.innerWidth();

                    // Calculate highlight size.
                    elementPosition.left = elementPosition.left - 3 - 3; // -3 for border and -(10/2) for h+10
                    elementPosition.top = elementPosition.top - 3 - 3;
                    highlight.offset(elementPosition);
                    highlight.height(h + 8);
                    highlight.width(w + 8);

                    // Show highligt.
                    highlight.show();
                    highlight.offset(elementPosition);

                    // Tell to navigator what was last highlighted element.
                    Diploma.Core.EventAggregator.Publish(Diploma.Core.Topics.Core_Pointer_Highlight, 0 /* Local */, element.get(0));
                    element.click();
                }
            };

            /**
            * Check if current cursor is.
            * @param cursorType Type of cursor.
            */
            Pointer.prototype.IsCursor = function (cursorType) {
                var isCursor = this._currentType === cursorType;
                return isCursor;
            };

            /**
            * Set temporary cursor and revert back when finished.
            * @param execute Callback function to execute under temporary cursor.
            */
            Pointer.prototype.Using = function (execute, temporaryCursorType) {
                var currentType = this._currentType;
                try  {
                    this._currentType = temporaryCursorType;
                    execute();
                } finally {
                    this._currentType = currentType;
                }
            };

            /** Get pointer element. */
            Pointer.prototype.GetPointer = function () {
                var div = this.GetDrawingSurface("body", "pointerSurface", function (div) {
                    var imageUrl = chrome.extension.getURL("/images/pointer40.png");
                    var css = {
                        "background-image": "url(" + imageUrl + ")",
                        "z-index": 2147483646 /* Level1 */
                    };
                    div.css(css);
                });
                return div;
            };

            /** Get highlight element. */
            Pointer.prototype.GetHighlight = function () {
                var div = this.GetDrawingSurface("body", "highlightSurface", function (div) {
                    var css = {
                        "z-index": 2147483646 /* Level1 */,
                        "border": "3px solid #0069cb",
                        "border-radius": "10px",
                        "box-shadow": "#B3B3B3 3px 3px 3px"
                    };
                    div.css(css);
                });
                return div;
            };

            /**
            * Get or create drawing surface if does not exsists.
            * @param parentSelector Parent element of drawing surface.
            * @param surfaceId Unique surface id.
            * @param setup Additional function to setup drawing surface.
            */
            Pointer.prototype.GetDrawingSurface = function (parentSelector, surfaceId, setup) {
                var div = $("#" + surfaceId);
                if (!div.length) {
                    var css = {
                        "position": "fixed",
                        "width": "40px",
                        "height": "40px",
                        "display": "none",
                        "pointer-events": "none",
                        "background-position": "center center",
                        "background-size": "100%",
                        "padding": "0px",
                        "margin": "0px"
                    };

                    $(parentSelector).append('<div id="' + surfaceId + '" />');
                    div = $("#" + surfaceId);
                    div.css(css);
                    setup(div);
                }
                return div;
            };

            /** Get instance of pointer (singletone). */
            Pointer.Default = function () {
                if (!Pointer._defaultPointer) {
                    Pointer._defaultPointer = new Pointer();
                }
                return Pointer._defaultPointer;
            };
            return Pointer;
        })();
        Core.Pointer = Pointer;

        /** Cursor types */
        var CursorType;
        (function (CursorType) {
            CursorType[CursorType["Pointer"] = 0] = "Pointer";
            CursorType[CursorType["ScreenTap"] = 1] = "ScreenTap";
            CursorType[CursorType["Highlight"] = 2] = "Highlight";
        })(CursorType || (CursorType = {}));

        /** ZIndex */
        var ZIndex;
        (function (ZIndex) {
            ZIndex[ZIndex["Level0"] = 2147483647] = "Level0";
            ZIndex[ZIndex["Level1"] = 2147483646] = "Level1";
            ZIndex[ZIndex["Level2"] = 2147483645] = "Level2";
        })(ZIndex || (ZIndex = {}));
    })(Diploma.Core || (Diploma.Core = {}));
    var Core = Diploma.Core;
})(Diploma || (Diploma = {}));
//# sourceMappingURL=Pointer.js.map
