// Copyright 2014 Miloš Vukov, BSD-style copyright and disclaimer apply
var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var Diploma;
(function (Diploma) {
    (function (Core) {
        /** Helper class for position and points */
        var PositionHelper = (function () {
            /** Initialize position helper. */
            function PositionHelper() {
                var _this = this;
                this.LoadFromStorage(); // Load mm-to-px from local storage.
                Diploma.Core.EventAggregator.Subscribe(Diploma.Core.Topics.Core_Calibration_Saved, function () {
                    // Load from storage when calibration was performed on other content script.
                    _this.LoadFromStorage();
                });
            }
            /** Load mm-to-px from local storage. */
            PositionHelper.prototype.LoadFromStorage = function () {
                var _this = this;
                chrome.storage.local.get(["mmToPxX", "mmToPxY"], function (items) {
                    _this._mmToPxX = items.mmToPxX == undefined ? (378 / 100) : items.mmToPxX;
                    _this._mmToPxY = items.mmToPxY == undefined ? (378 / 100) : items.mmToPxY;
                });
            };

            /**
            * Get document 2D point from LeapMotion 3D point.
            * @param lmPointMm LeapMotion point.
            */
            PositionHelper.prototype.LeapMotionToDocumentPoint = function (lmPointMm) {
                var dX = this.LMtoD_X(this.MilimeterToPixelX(lmPointMm.X));
                var dY = this.LMtoD_Y(this.MilimeterToPixelY(lmPointMm.Y));
                var dZ = this.MilimeterToPixelY(0);
                var documentPoint = new Point3([dX, dY, dZ]);
                return documentPoint;
            };

            /**
            * Convert X mm to px.
            * @param milimeter X value in milimeter.
            */
            PositionHelper.prototype.MilimeterToPixelX = function (milimeter) {
                // Debug
                //var mmToPx = 356 / 100;     // 1680*1050
                //var mmToPx = 580 / 100;     // 1920*1280
                //var mmToPx = 378 / 100;     // browser calculated
                //var mmToPx = 352 / 100;     // with calibration
                return milimeter * this._mmToPxX;
            };

            /**
            * Convert Y mm to px.
            * @param milimeter Y value in milimeter.
            */
            PositionHelper.prototype.MilimeterToPixelY = function (milimeter) {
                // Debug
                //var mmToPx = 356 / 100;     // 1680*1050
                //var mmToPx = 580 / 100;     // 1920*1280
                //var mmToPx = 378 / 100;     // browser calculated
                //var mmToPx = 336 / 100;     // with calibration
                return milimeter * this._mmToPxY;
            };

            /**
            * Translate LeapMotion X coordinate to document X coordinate.
            * @param lmPointPx LeapMotion X coordinate in px.
            */
            PositionHelper.prototype.LMtoD_X = function (lmPointPx) {
                var LMtoS_X = lmPointPx + screen.width / 2;
                return LMtoS_X - window.screenLeft + Diploma.Core.FineCalibration.Default().DiffX;
            };

            /**
            * Translate LeapMotion Y coordinate to document Y coordinate.
            * @param lmPointPx LeapMotion Y coordinate in px.
            */
            PositionHelper.prototype.LMtoD_Y = function (lmPointPx) {
                var LMtoS_Y = screen.height - lmPointPx;
                return LMtoS_Y - window.screenTop + Diploma.Core.FineCalibration.Default().DiffY;
            };

            /** Get instance of position helper (singleton). */
            PositionHelper.Default = function () {
                if (!PositionHelper._defaultPositionHelper) {
                    PositionHelper._defaultPositionHelper = new PositionHelper();
                }
                return PositionHelper._defaultPositionHelper;
            };
            return PositionHelper;
        })();

        /** 3D point. */
        var Point3 = (function () {
            /** Initialize Point3 from leapjs 3D point. */
            function Point3(point) {
                this.X = point[0];
                this.Y = point[1];
                this.Z = point[2];
            }
            /** Get document point ( from LeapMotion point ). */
            Point3.prototype.ToPointD = function () {
                if (!this._pointD) {
                    this._pointD = PositionHelper.Default().LeapMotionToDocumentPoint(this);
                }
                return this._pointD;
            };
            return Point3;
        })();
        Core.Point3 = Point3;

        /** Normal/direction vector */
        var NormalVector = (function (_super) {
            __extends(NormalVector, _super);
            function NormalVector() {
                _super.apply(this, arguments);
            }
            /** Vector pointing to left in xy plane. */
            NormalVector.prototype.Left = function () {
                return this.InAngle(2.6, 4.0);
            };

            NormalVector.prototype.ToPointD = function () {
                throw new Diploma.Core.OperationNotSupportedError();
            };

            /**
            * Check if it is in angle.
            * @param from Angle in radian.
            * @param to Angle in radian.
            */
            NormalVector.prototype.InAngle = function (from, to) {
                var vec2Lib = Leap.glMatrix.vec2;
                var vec2Len = vec2Lib.length(vec2Lib.fromValues(this.X, this.Y));
                var angle = Math.acos(this.X / vec2Len);
                return angle >= from && angle <= to;
            };
            return NormalVector;
        })(Point3);
        Core.NormalVector = NormalVector;
    })(Diploma.Core || (Diploma.Core = {}));
    var Core = Diploma.Core;
})(Diploma || (Diploma = {}));
//# sourceMappingURL=Vector3.js.map
