// Copyright 2014 Miloš Vukov, BSD-style copyright and disclaimer apply
var Diploma;
(function (Diploma) {
    (function (Motion) {
        var Core = Diploma.Core;

        /** Switch to tab. */
        var ChangeTab = (function () {
            /** Initialize change tab. */
            function ChangeTab() {
                this._debouncedSwitchToPreviousTab = _.debounce(function () {
                    this.SwitchToPreviousTab();
                }, 200, true);
            }
            /** Initialize chnage tab. */
            ChangeTab.prototype.Initialize = function () {
                var _this = this;
                var controller = new Leap.Controller({ enableGestures: true, background: true });
                var swiper = controller.gesture('swipe');
                swiper.stop(function (gesture) {
                    _this.Action(gesture);
                });

                controller.connect();
            };

            /**
            * Change tab if gesture is detected.
            * @param gesture LeapMotion swipe gesture.
            */
            ChangeTab.prototype.Action = function (gesture) {
                if (this.Detected(gesture))
                    this._debouncedSwitchToPreviousTab();
            };

            /**
            * Check gesture.
            * @param gesture LeapMotion swipe gesture.
            */
            ChangeTab.prototype.Detected = function (gesture) {
                if (gesture && gesture.gestures && gesture.lastFrame.hands.length == 1) {
                    // Check palm direction.
                    var handVector = new Diploma.Core.NormalVector(gesture.lastFrame.hands[0].palmNormal);
                    if (handVector.Left() != true)
                        return false;

                    // Check gesture direction.
                    var lastIndex = gesture.gestures.length - 1;
                    var vector = new Diploma.Core.NormalVector(gesture.gestures[lastIndex].direction);
                    return vector.Left() == true;
                }
                return false;
            };

            /** Switch to previous tab. */
            ChangeTab.prototype.SwitchToPreviousTab = function () {
                // Get all tabs in current window.
                chrome.tabs.query({ windowId: chrome.windows.WINDOW_ID_CURRENT }, function (tabs) {
                    if (tabs && tabs.length > 1) {
                        var lastIndex = (tabs.length - 1);
                        for (var index = 0; index < tabs.length; index++) {
                            var tab = tabs[index];
                            if (tab.active) {
                                if (tab.index !== 0) {
                                    // Switch to previous tab
                                    chrome.tabs.update(tabs[index - 1].id, { active: true });
                                    return;
                                } else {
                                    // Switch to last tab
                                    chrome.tabs.update(tabs[lastIndex].id, { active: true });
                                    return;
                                }
                            }
                        }
                    }
                });
            };
            return ChangeTab;
        })();
        Motion.ChangeTab = ChangeTab;
    })(Diploma.Motion || (Diploma.Motion = {}));
    var Motion = Diploma.Motion;
})(Diploma || (Diploma = {}));
//# sourceMappingURL=ChangeTab.js.map
