// Copyright 2014 Milo Vukov, BSD-style copyright and disclaimer apply

module Diploma.Motion
{
    import Core = Diploma.Core;

    /** Switch to tab. */
    export class ChangeTab implements IInitializer
    {
        private _debouncedSwitchToPreviousTab;

        /** Initialize change tab. */
        constructor()
        {
            this._debouncedSwitchToPreviousTab = _.debounce(function () { this.SwitchToPreviousTab() }, 200, true);
        }

        /** Initialize chnage tab. */
        public Initialize()
        {
            var controller = new Leap.Controller({ enableGestures: true, background: true });
            var swiper = controller.gesture('swipe');
            swiper.stop((gesture) =>
            {
                this.Action(gesture);
            });

            controller.connect();
        }

        /**
         * Change tab if gesture is detected.
         * @param gesture LeapMotion swipe gesture.
         */
        private Action(gesture: Gesture): void
        {
            if (this.Detected(gesture))
                this._debouncedSwitchToPreviousTab();
        }

        /**
         * Check gesture.
         * @param gesture LeapMotion swipe gesture.
         */
        private Detected(gesture: Gesture): boolean
        {
            if (gesture && gesture.gestures && gesture.lastFrame.hands.length == 1)
            {
                // Check palm direction.
                var handVector = new Diploma.Core.NormalVector(gesture.lastFrame.hands[0].palmNormal);
                if (handVector.Left() != true)
                    return false;

                // Check gesture direction.
                var lastIndex = gesture.gestures.length - 1;
                var vector = new Diploma.Core.NormalVector(gesture.gestures[lastIndex].direction);
                return vector.Left() == true;
            }
            return false;
        }

        /** Switch to previous tab. */
        private SwitchToPreviousTab()
        {
            // Get all tabs in current window.
            chrome.tabs.query({ windowId: chrome.windows.WINDOW_ID_CURRENT }, (tabs) =>
            {
                if (tabs && tabs.length > 1)
                {
                    var lastIndex = (tabs.length - 1);
                    for (var index = 0; index < tabs.length; index++)
                    {
                        var tab = tabs[index];
                        if (tab.active)
                        {
                            if (tab.index !== 0) // Is not first tab.
                            {
                                // Switch to previous tab
                                chrome.tabs.update(tabs[index - 1].id, { active: true });
                                return;
                            }
                            else
                            {
                                // Switch to last tab
                                chrome.tabs.update(tabs[lastIndex].id, { active: true });
                                return;
                            }
                        }
                    }
                }
            });
        }
    }
}
