// Copyright 2014 Milo Vukov, BSD-style copyright and disclaimer apply

module Diploma.Motion
{
    import Core = Diploma.Core;

    /** Activate hyperlink */
    export class Navigate implements IInitializer
    {
        private _debouncedExecuteClick;
        private _highlightedElement: JQuery;

        /** Initialize navigate. */
        constructor()
        {
            // Initialize debounced execute click.
            this._debouncedExecuteClick = _.debounce(
                function (gesture: Gesture, lastHighlightedElement: JQuery)
                {
                    this.ExecuteClick(gesture, lastHighlightedElement);
                }, 100, true);
            // Listen to clickable element selected messages.
            Core.EventAggregator.Subscribe(Core.Topics.Core_Pointer_Highlight, (element) => { this._highlightedElement = $(element); });
        }

        /**
         * Intialize navigate.
         * @param controller Initialized LeapMotion controller.
         */
        public Initialize(controller: Controller)
        {
            // Listen to screen tap gesture.
            var taper = controller.gesture('screenTap');
            taper.stop((gesture) =>
            {
                this.Action(gesture);
            });
        }

        /**
         * Execute click on element.
         * @param gesture LeapMotion screen tap gesture.
         */
        private Action(gesture: Gesture): void
        {
            if (this.Detected(gesture))
                this._debouncedExecuteClick(gesture, this._highlightedElement);
        }

        /**
         * Check if gesture is detected.
         * @param gesture LeapMotion screen tap gesture.
         */
        private Detected(gesture: Gesture): boolean
        {
            var detected = gesture && gesture.gestures
                && gesture.gestures.length > 0
                && this._highlightedElement != undefined; // Clickable element must be highlighted.
            return detected;
        }

        /**
         * Execute click on element.
         * @param clickableElement JQuery element.
         */
        private ExecuteClick(clickableElement: JQuery)
        {
            // Show animation
            Core.Pointer.Default().ScreenTap();
            // Execute click
            clickableElement[0].click();
        }
    }
}
